/*
   _===_ _           _                  ______ _     _             _ _ _
  |  ___| |         | |                 | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|

  Puissance-mètre RF basé sur le AD8310 et le AD8318.
  Compilé pour le STM32F103C8. 
  Utilise un Module LCD 16x2 classique (HD44780) équipé d'une plaquette d'interface I2C basée sur le mux i2c PCF8574.
  Électro-Bidouilleur, Novembre 2017
 */
 
#include <Wire.h>   // La librairie I2C du STM32
/*    Définition des broches du I2C dans la librairie wire.h: 
       SDA -> PA7
       SCL -> PA6    */
       
#include <LiquidCrystal_PCF8574.h>
/* Cette librairie a été installée à partir du menu "Sketch/Include Librairy/Manage Librairies..."
   Elle se nomme LiquidCrystal_PCF8574. Plus d'info au http://www.mathertel.de/Arduino/LiquidCrystal_PCF8574.aspx 
   Par Mathias Hertel. */

#define addr_LCD 0x27       // L'adresse I2C du LCD

LiquidCrystal_PCF8574 lcd(addr_LCD);  // Configure l'adresse I2C du LCD à 0x27

// Constantes
const int voie1_entree = PA0;       // Broche d'entrée du ADC, voie 1
const int voie2_entree = PA1;       // Broche d'entrée du ADC, voie 2
const int bouton1 = PA12;           // Broche d'entrée du bouton poussoir
// La definition des constantes pentes et decalage utilisees dans le calcul de la puissance
  // Voie 1
const char num_gamme1 = 5;             // Le nombre de bandes de paramètres pente/décalage de la Voie1 1
const char *gamme1_txt[num_gamme1] =    {"0-100 MHz" ,"100-200 MHz","200-300 MHz","300-400 MHz","400-440 MHz"};   // Texte des bandes
const float gamme1_pente[] =            {   0.0237   ,   0.02325   ,   0.023     ,   0.0227    ,   0.0228    };   // Pentes tirées de la datasheet
const float gamme1_decalage[] =         {    -95.6   ,   -95.1     ,   -94.4     ,    -93      ,    -91      };   // Décalages tirés de la datasheet
 // Voie 2
const char num_gamme2 = 6;             // Le nombre de bandes de paramètres pente/décalage de la Voie 2
const char *gamme2_txt[num_gamme2] =    {  "0-1 GHz" , "1-2 GHz"   ,  "2,2 GHz"  ,  "3,6 GHz"  ,  "5,8 GHz"  ,  "8 GHz"  };   // Texte des bandes
const float gamme2_pente[] =            {  -0.0245   ,  -0.0245    ,  -0.0244    ,  -0.0243    ,   -0.0243   ,   -0.023   };  // Pentes tirées de la datasheet
const float gamme2_decalage[] =         {     22     ,    21.2     ,    19.6     ,    19.8     ,      25     ,    37     };   // Décalages tirées de la datasheet
 
// Variables
unsigned char cntr;                   // Compteur servant à calculer le moyenne des lectures
unsigned char cntr_delai = 0;         // Compteur de temps de délai dans les menus de paramètres
unsigned char cntr_delai_sortie = 0;  // Compteur de temps de délai pour la sortie des menus de paramètres
unsigned char voie_select = 1;        // Voie sélectionnée (active)
float voie1_lecture;                  // Valeur lue sur l'ADC
float voie2_lecture;                  //    "
float voie1_puissance;                // Valeur calculée de puissance en dBm
float voie2_puissance;                //    "
unsigned char gamme1 = 0;             // Gamme sélectionnée sur la voie 1
unsigned char gamme2 = 0;             // Gamme sélectionnée sur la voie 2



// Fonction d'initialisation
void setup() 
{
  // Déclaration des mode de broches du ADC:
  pinMode(voie1_entree, INPUT_ANALOG);
  pinMode(voie2_entree, INPUT_ANALOG);
  pinMode(bouton1, INPUT_PULLUP);

  // Initialisation du port i2c du LCD
  Wire.begin();                 // Initialisation de la communication I2C avec le LCD

  // Définir et initialiser le LCD
  lcd.begin(16, 2);             // Définir et initialiser le LCD comme ayant 16 colonnes et 2 lignes
  lcd.setBacklight(255);        // Allumer le rétro-éclairage
  lcd.home();                   // positionner le curseur à la première case
  lcd.clear();                  // Effacer l'écran

  // Affichage du logo d'allumage
  lcd.setCursor(0, 0);          // positionner le curseur
  lcd.print("Electro-");        // Afficher le texte entre apostrophes
  lcd.setCursor(4, 1);          // positionner le curseur
  lcd.print("Bidouilleur!");    // Afficher le texte entre apostrophes
  delay(2000);                  // Pause de 2 secondes

  // Affichage du texte permanent
  lcd.home();                   // positionner le curseur à la première case
  lcd.clear();                  // Effacer l'écran
  lcd.setCursor(0, 0);          // positionner le curseur
  lcd.print("1: ");             // Afficher le texte entre apostrophes
  lcd.setCursor(10, 0);         // positionner le curseur
  lcd.print("dBm");             // Afficher le texte entre apostrophes
  lcd.setCursor(0, 1);          // positionner le curseur
  lcd.print("2: ");             // Afficher le texte entre apostrophes
  lcd.setCursor(10, 1);         // positionner le curseur
  lcd.print("dBm");             // Afficher le texte entre apostrophes
}

// Boucle Perpétuelle
void loop()
{ 
  if (!digitalRead(bouton1))    // Bouton pesé pour accéder au menu?
  {   // oui
    for(cntr_delai=1; cntr_delai<100; cntr_delai++)     // Compter si bouton pesé pendant 1 seconde
    {
      delay(10);                            // 10 milliseconde x 100 = 1 seconde                  
      if (digitalRead(bouton1)) break;      // bouton relache avant 1 seconde, on ne fait rien, sortir de la boucle
    } 
    if (cntr_delai >= 100)                  // Le bouton a été pesé pendent une deconde, entrée dans le menu de config.
    {
      lcd.home();                           // positionner le curseur à la première case
      lcd.clear();                          // Effacer l'écran
      if (voie_select == 1)                 // Voie 1 active?
      {  // Oui
        lcd.setCursor(0, 0);                // positionner le curseur
        lcd.print("1: ");                   // Afficher le texte de voie 1 
        lcd.print(gamme1_txt[gamme1]);      // Afficher la gamme courante 
      }
      else 
      {
        lcd.setCursor(0, 1);                // positionner le curseur
        lcd.print("2: ");                   // Afficher le texte de voie 2 
        lcd.print(gamme2_txt[gamme2]);      // Afficher la gamme courante 
      }
      while (!digitalRead(bouton1));        // Attendre que le bouton soit relâché
      cntr_delai = 1;                       // Ré-initialiser le compteur 
      cntr_delai_sortie = 1;                // Ré-initialiser le compteur de délai de sortie des menus
      while (cntr_delai_sortie < 200)       // boucler tant que le Delai pour sortir des menus n'est pas atteint
      { 
        delay(10);                          // 10 milliseconde x 200 = 2 seconde            
        cntr_delai_sortie++;                // incrémenter le compteur de délai de sortie des menus
        if (!digitalRead(bouton1))          // Bouton pesé?
        {   // Oui
          for(cntr_delai=1; cntr_delai<100; cntr_delai++)     // Compter si bouton pesé pendant 1 seconde     
          {
            delay(10);                      // 10 milliseconde x 100 = 1 seconde            
            if (digitalRead(bouton1)) break; // bouton relache avant 1 seconde? Sortir de la boucle
          }
          if (cntr_delai >= 100)  // Bouton pesé pendant 1 seconde, Mode de selection de voie invoqué
          {
            lcd.clear();                          // Effacer l'écran
            if (voie_select == 1)                 // Voie 1 active?
            {   // Oui
              voie_select = 2;                    // Mettre la voie 2 active
              lcd.setCursor(0, 1);                // Positionner le curseur
              lcd.print("2: ");                   // Afficher le texte de la voie 2
              lcd.print(gamme2_txt[gamme2]);      // Afficher la gamme courante de la voie 2
            }
            else    // Voie 2 active
            {
              voie_select = 1;                    // Mettre la voie 1 active
              lcd.setCursor(0, 0);                // Positionner le curseur
              lcd.print("1: ");                   // Afficher le texte de la voie 1
              lcd.print(gamme1_txt[gamme1]);      // Afficher la gamme courante de la voie 1
            }
            while (!digitalRead(bouton1));        // Attendre que le bouton soit relache
          }
          else  // bouton pese moins de 1 seconde, changement de gamme invoqué
          { 
            if (voie_select == 1)                 // Voie 1 active?
            {
              gamme1++;                                     // Incrémenter la gamme de la voie 1
              if (gamme1 > (num_gamme1-1)) gamme1 = 0;      // boucler si nécessaire le choix de gamme à la première gamme    
              lcd.clear();                                  // Effacer l'écran
              lcd.setCursor(0, 0);                          // Positionner le curseur
              lcd.print("1: ");                             // Afficher le texte de la voie 1
              lcd.print(gamme1_txt[gamme1]);                // Afficher la gamme courante de la voie 1
            }
            else 
            {
              gamme2++;                                     // Incrémenter la gamme de la voie 2
              if (gamme2 > (num_gamme2-1)) gamme2 = 0;      // boucler si nécessaire le choix de gamme à la première gamme    
              lcd.clear();                                  // Effacer l'écran
              lcd.setCursor(0, 1);                          // Positionner le curseur
              lcd.print("2: ");                             // Afficher le texte de la voie 2
              lcd.print(gamme2_txt[gamme2]);                // Afficher la gamme courante de la voie 2
            }
            cntr_delai_sortie = 1;                          // Ré-initialiser le compteur de délai de sortie des menus                          
          }
        }
      }
      // Sortie des menus. Afficher le texte permanent
      lcd.clear();              // Effacer l'écran
      lcd.setCursor(0, 0);      // positionner le curseur
      lcd.print("1: ");         // Afficher le texte entre apostrophes
      lcd.setCursor(10, 0);     // positionner le curseur
      lcd.print("dBm");         // Afficher le texte entre apostrophes
      lcd.setCursor(0, 1);      // positionner le curseur
      lcd.print("2: ");         // Afficher le texte entre apostrophes
      lcd.setCursor(10, 1);     // positionner le curseur
      lcd.print("dBm");         // Afficher le texte entre apostrophes
    }
  }
  // Faire une moyenne des lectures de ADC pour eliminer le bruit
  for (cntr = 1; cntr <= 100; cntr++)
  {
    voie1_lecture = ((voie1_lecture * (cntr - 1)) + analogRead(voie1_entree)) / cntr;   // Lecture des entrées DAC à répétition pour faire une moyenne
    voie2_lecture = ((voie2_lecture * (cntr - 1)) + analogRead(voie2_entree)) / cntr;   //     "
  }
  // Calculer la puissance en dBm (par defaut)
    // Obtenu de la datasheet du AD8310:
    //    P_in = (V_out / Vslope) + P_offset
    //       et V_slope = 0.0237 V/dB
    //       et P_offset = -95 dB
    // Obtenu de la datasheet du AD8318:
    //    P_in = (V_out / Vslope) + P_offset
    //       et V_slope = -0.0245 V/dB
    //       et P_offset = +22 dB
    // Aussi, compte tenu du ADC de 12 bits de 3.3 V:
    //     V_out = (ADC / 4095) * V_ref
    //       et V_ref = 3.3 V
  
  voie1_puissance = ((voie1_lecture / 4095 * 3.3) / gamme1_pente[gamme1]) + gamme1_decalage[gamme1];   // Calcul de puissance en fonction du mot de DAC lu.
  voie2_puissance = ((voie2_lecture / 4095 * 3.3) / gamme2_pente[gamme2]) + gamme2_decalage[gamme2];  //     "

  // Afficher les résultats
  lcd.setCursor(3, 0);                                // Positionner le curseur
  if (voie1_puissance < -70) lcd.print("input?");     // Si le signal est trop faible, afficher "input?"
  else 
  {
    lcd.print("      ");                              // Effacer les caractères
    lcd.setCursor(3, 0);                              // Positionner le curseur
    lcd.print(voie1_puissance);                       // Afficher la puissance
  }
  lcd.setCursor(3, 1);                                // Positionner le curseur
  if (voie2_puissance < -62) lcd.print("input?");     // Si le signal est trop faible, afficher "input?"
  else 
  {
    lcd.print("      ");                              // Effacer les caractères
    lcd.setCursor(3, 1);                              // Positionner le curseur
    lcd.print(voie2_puissance);                       // Afficher la puissance
  }
  delay(100);                                         // Délai pour ralentir l'affichage, sinon trop rapide pour voir les chiffres qui varient
}
